/***********************************************************************
 * Linotte                                                             *
 * Version release date : March 12, 2009                               *
 * Author : Mounes Ronan ronan.mounes@amstrad.eu                        *
 *                                                                     *
 *     http://langagelinotte.free.fr                                   *
 *                                                                     *
 * This code is released under the GNU GPL license, version 2 or       *
 * later, for educational and non-commercial purposes only.            *
 * If any part of the code is to be included in a commercial           *
 * software, please contact us first for a clearance at                *
 *   ronan.mounes@amstrad.eu                                            *
 *                                                                     *
 *   This notice must remain intact in all copies of this code.        *
 *   This code is distributed WITHOUT ANY WARRANTY OF ANY KIND.        *
 *   The GNU GPL license can be found at :                             *
 *           http://www.gnu.org/copyleft/gpl.html                      *
 *                                                                     *
 ***********************************************************************/

package org.linotte.frame.latoile;

import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.HeadlessException;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.WindowListener;
import java.awt.image.BufferedImage;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.JApplet;
import javax.swing.JFrame;

import org.alize.kernel.AKRuntime;
import org.linotte.frame.atelier.Chargement;
import org.linotte.moteur.entites.PrototypeGraphique;
import org.linotte.moteur.outils.Ressources;
import org.linotte.moteur.xml.Linotte;
import org.linotte.moteur.xml.Version;

import console.Jinotte;

@SuppressWarnings("serial")
public class Applet extends JApplet implements LaToile {

	private AKRuntime runtime;

	class AppletRunnable implements Runnable {

		LaToile toile;

		AppletRunnable(LaToile ptoile) {
			toile = ptoile;
		}

		public void run() {
			String livre = getParameter("livre");
			String[] param = { "-d", "-v", "-a", livre == null ? "" : livre };
			try {
				Jinotte jinotte = new Jinotte(toile);
				jinotte.preparation(param);
				jinotte.lecture();
			} catch (Error e) {
				e.printStackTrace();
				setMessage(e.getMessage());
			} catch (Exception e) {
				e.printStackTrace();
				setMessage(e.getMessage());
			}

		}

	}

	private LaToileListener panelLaToile;

	private String message = null;

	protected Chargement chargement = null;

	private Image fond = null;

	private boolean barre;

	private boolean atelier;

	private static Linotte linotte;

	public void setMessage(String m) {
		message = m;
		if (panelLaToile != null)
			panelLaToile.setVisible(message == null);
		repaint();
	}

	@Override
	public void init() {
		super.init();
		//x-default-cursor.gif
		try {
			Cursor monCurseur = createCursorFromIcon(ImageIO.read(Ressources.getFlux("x-default-cursor.gif")), "X curseur");
			setCursor(monCurseur);
		} catch (HeadlessException e) {
			e.printStackTrace();
		} catch (IndexOutOfBoundsException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		fond = Ressources.getImageIcon("chargement.png").getImage();
		if (linotte == null) {
			Frame frame = null;
			Component parent = getParent();
			while ((parent != null) && !(parent instanceof Frame)) {
				parent = parent.getParent();
			}
			if (parent instanceof Frame)
				frame = (Frame) parent;

			chargement = new Chargement(frame);
			panelLaToile = new LaToileListener(this);
			panelLaToile.init();

			Toile.initApplet(this);
			LaToile toile = this;
			add(panelLaToile);
			setMessage("Ouverture du livre ...");
			Runnable thread = new AppletRunnable(toile);
			new Thread(thread).start();
		} else {
			setMessage("Une applet Linotte tourne déjà !");
		}
	}

	@Override
	public void paint(Graphics g) {
		super.paint(g);
		if (message != null) {
			g.drawImage(fond, 0, 0, null);
			int d = 120;
			g.drawString(message, 20, 20 + d);
			g.drawString("VM Linotte " + Version.getVersion(), 20, 40 + d);
		}
	}

	@Override
	public void destroy() {
		if (linotte != null) {
			//System.out.println("Début de la destruction de l'environnement Linotte.");
			try {
				//TODO Alizé
				runtime.stopAll();
			} finally {
				panelLaToile = null;
				message = null;
				chargement = null;
				fond = null;
				linotte = null;
				System.gc();
				System.out.println("Fin de la destruction de l'environnement Linotte.");
			}
		}
	}

	/**
	 * Create a cursor than can be used with any component from an icon.
	 * The icon can be any kind of image that can be read with ImageIO (jpg, png, gif, bmp)
	 * and is specified by its path.
	 * http://www.developpez.net/forums/d425877/java/interfaces-graphiques-java/awt-swing/changer-curseur-defaut/
	 * @param imagePath Name of the image file to read
	 * @param cursorName name of the cursor for later use
	 * @return the Cursor that can be applied to a component.
	 */
	public static Cursor createCursorFromIcon(BufferedImage icon, String cursorName) {

		// The window system may not be able to use the specified image.
		// Some window system are not able to display cursors with certain size.
		// We then need to resize the image to the compatible dims.
		Dimension compatibleDims = Toolkit.getDefaultToolkit().getBestCursorSize(icon.getWidth(), icon.getHeight());

		// create a new image with a compatible dimentions
		Image compatibleIcon = null;
		if (compatibleDims.width == icon.getWidth() || compatibleDims.height == icon.getHeight()) {
			// no need for resize just use the icon
			compatibleIcon = icon;
		} else if (icon.getWidth() < compatibleDims.width && icon.getHeight() < compatibleDims.height) {
			// the icon is smaller than the compatible size
			// We create a new image with a compatible size and copy the icon on the top left corner.
			// The remaining part of the icon is kept transparent, so the cursor rendering
			// is actually unchanged
			BufferedImage compatibleIconB = new BufferedImage(compatibleDims.width, compatibleDims.height, BufferedImage.TYPE_INT_ARGB);

			// by default set the image to transparent
			for (int i = 0; i < compatibleIconB.getWidth(); i++) {
				for (int j = 0; j < compatibleIconB.getHeight(); j++) {
					compatibleIconB.setRGB(i, j, 0);
				}
			}

			// then copy the icon in the top left part
			for (int i = 0; i < icon.getWidth(); i++) {
				for (int j = 0; j < icon.getHeight(); j++) {
					// we're lucky since getRGB return the value as ARGB
					compatibleIconB.setRGB(i, j, icon.getRGB(i, j));
				}
			}

			compatibleIcon = compatibleIconB;
		} else {
			// the icon is larger than the compatible size
			// we then need to reduce it
			compatibleIcon = icon.getScaledInstance(compatibleDims.width, compatibleDims.height, Image.SCALE_SMOOTH);
		}

		// finally create the Cursor
		// the second parameter is quite obscure, the java doc does not help very much. But using
		// (0, 0) as value works fine so let's use this.
		return Toolkit.getDefaultToolkit().createCustomCursor(compatibleIcon, new Point(0, 0), cursorName);
	}

	public void addWindowListener(WindowListener closeListener) {
	}

	public void dispose() {
	}

	public JFrame getFrameParent() {
		return null;
	}

	public LaToileListener getPanelLaToile() {
		return panelLaToile;
	}

	public boolean isAffichable() {
		return true;
	}

	public boolean isAlwaysOnTop() {
		return false;
	}

	public boolean isFocused() {
		return true;
	}

	public boolean isUndecorated() {
		return false;
	}

	public void pack() {
	}

	public void setAlwaysOnTop(boolean newvalueTop) {
	}

	public void setLocationRelativeTo(Component object) {
	}

	public void setResizable(boolean b) {
	}

	public void setTitle(String titre) {
	}

	public void setUndecorated(boolean newvalue) {
	}

	public boolean isAtelier() {
		return atelier;
	}

	public boolean isBarreDesTaches() {
		return barre;
	}

	public void setAtelier(boolean patelier) {
		atelier = patelier;
	}

	public void setBarreDesTaches(boolean pbarre) {
		barre = pbarre;
	}

	public Linotte getInterpreteur() {
		return linotte;
	}

	public void setInterpreteur(Linotte pinterpreteur) {
		linotte = pinterpreteur;
	}

	public void setRuntime(AKRuntime runtime) {
		this.runtime = runtime;
	}

	public AKRuntime getRuntime() {
		return runtime;
	}

	@Override
	public PrototypeGraphique getElementDragAndDrop() {
		return panelLaToile.getElementDragAndDrop();
	}
}

